@@ -7,7 +7,7 @@ import numpy
 import warnings
 import scipy.sparse
 import numpy.fft
-import scipy.signal
+#import scipy.signal
 import scipy.linalg
 import scipy.special
 from functools import wraps as _wraps
@@ -277,4 +277,4 @@ class NUFFT:
 #                 'hsa': self._reset_sense_host,
 #                 'hsa_legacy': self._reset_sense_host}
 #         return func.get(self.processor)(*args, **kwargs)
-    
\ No newline at end of file
+    
