@@ -29,8 +29,10 @@ def create_kernel_sets(API):
                         cDiff() + 
                         cSqrt() + 
                         cAnisoShrink() +  
+                        cRealShrink() +
                         cSpmv() + 
                         cSpmvh() + 
+                        cCopyColumn()+
                         cHadamard())
 #     if 'cuda' is API:
 #         print('Select cuda interface')
@@ -59,7 +61,22 @@ def cMultiplyScalar():
         };           
         """
     return code_text
-
+def cCopyColumn():
+    code_text = """
+    KERNEL void cCopyColumn( 
+            const unsigned int n, // start
+            const unsigned int m, // step
+             GLOBAL_MEM  const float2 *CX,
+             GLOBAL_MEM             float2 *CY)
+    {
+    // Copy x to y: y = x;
+    //CX: input array (float2)
+    // CY output array (float2)
+    unsigned long gid=get_global_id(0);  
+    CY[gid]=CX[n+m*gid];
+    };
+    """  
+    return code_text    
 def cCopy():
     """
     Return the kernel source for cCopy
@@ -1011,6 +1028,28 @@ def cDiff():
         };
         """
     return R
+
+def cRealShrink():
+    """
+    Return the kernel source of xAnisoShrink
+    """
+    R="""
+    KERNEL void xRealShrink(const  float2 threshold,
+                                    GLOBAL_MEM const float2 *indata,
+                                    GLOBAL_MEM float2 *outdata)
+    {
+    const unsigned int gid =  get_global_id(0); 
+    float2 tmp; // temporay register
+    tmp = indata[gid];
+    //float zero = 0.0;
+    //tmp.x=sign(tmp.x)*max(fabs(tmp.x)-threshold.x, zero); 
+    //tmp.y=sign(tmp.y)*max(fabs(tmp.y)-threshold.y, zero); 
+    tmp.x =  (tmp.x > threshold.x)*(tmp.x - threshold.x) ; //+ (tmp.x <= - threshold.x)*(tmp.x + threshold.x);
+    tmp.y =  (tmp.y > threshold.x)*(tmp.y - threshold.x) ; //+ (tmp.y <= - threshold.x)*(tmp.y + threshold.x);
+    outdata[gid]=tmp;
+    };
+    """
+    return R
 def cAnisoShrink():
     """
     Return the kernel source of cAnisoShrink
@@ -1026,8 +1065,8 @@ def cAnisoShrink():
     //float zero = 0.0;
     //tmp.x=sign(tmp.x)*max(fabs(tmp.x)-threshold.x, zero); 
     //tmp.y=sign(tmp.y)*max(fabs(tmp.y)-threshold.y, zero); 
-    tmp.x =  (tmp.x > threshold.x)*(tmp.x - threshold.x) ;//+ (tmp.x < - threshold.x)*(tmp.x + threshold.x);
-    tmp.y =  (tmp.y > threshold.x)*(tmp.y - threshold.x) ;//+ (tmp.y < - threshold.x)*(tmp.y + threshold.x);
+    tmp.x =  (tmp.x > threshold.x)*(tmp.x - threshold.x) + (tmp.x < - threshold.x)*(tmp.x + threshold.x);
+    tmp.y =  (tmp.y > threshold.x)*(tmp.y - threshold.x) + (tmp.y < - threshold.x)*(tmp.y + threshold.x);
     outdata[gid]=tmp;
     };
     """
